import { platform } from 'os';
import slash from 'slash';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __spreadArrays() {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
}

var VOLUME = /^([A-Z]:)/i;
var IS_WINDOWS = platform() === 'win32';
var noop = function () { return null; };
function matches(pattern, importee) {
    if (pattern instanceof RegExp) {
        return pattern.test(importee);
    }
    if (importee.length < pattern.length) {
        return false;
    }
    if (importee === pattern) {
        return true;
    }
    var importeeStartsWithKey = importee.indexOf(pattern) === 0;
    var importeeHasSlashAfterKey = importee.substring(pattern.length)[0] === '/';
    return importeeStartsWithKey && importeeHasSlashAfterKey;
}
function normalizeId(id) {
    if (typeof id === 'string' && (IS_WINDOWS || VOLUME.test(id))) {
        return slash(id.replace(VOLUME, ''));
    }
    return id;
}
function getEntries(_a) {
    var entries = _a.entries;
    if (!entries) {
        return [];
    }
    if (Array.isArray(entries)) {
        return entries;
    }
    return Object.entries(entries).map(function (_a) {
        var key = _a[0], value = _a[1];
        return { find: key, replacement: value };
    });
}
function getCustomResolver(_a, options) {
    var customResolver = _a.customResolver;
    if (typeof customResolver === 'function') {
        return customResolver;
    }
    if (customResolver && typeof customResolver.resolveId === 'function') {
        return customResolver.resolveId;
    }
    if (typeof options.customResolver === 'function') {
        return options.customResolver;
    }
    if (options.customResolver && typeof options.customResolver.resolveId === 'function') {
        return options.customResolver.resolveId;
    }
    return null;
}
function alias(options) {
    if (options === void 0) { options = {}; }
    var entries = getEntries(options);
    if (entries.length === 0) {
        return {
            name: 'alias',
            resolveId: noop
        };
    }
    return {
        name: 'alias',
        buildStart: function (inputOptions) {
            var _this = this;
            return Promise.all(__spreadArrays(entries, [options]).map(function (_a) {
                var customResolver = _a.customResolver;
                return customResolver &&
                    typeof customResolver === 'object' &&
                    typeof customResolver.buildStart === 'function' &&
                    customResolver.buildStart.call(_this, inputOptions);
            })).then(function () {
                // enforce void return value
            });
        },
        resolveId: function (importee, importer) {
            var importeeId = normalizeId(importee);
            var importerId = normalizeId(importer);
            // First match is supposed to be the correct one
            var matchedEntry = entries.find(function (entry) { return matches(entry.find, importeeId); });
            if (!matchedEntry || !importerId) {
                return null;
            }
            var updatedId = normalizeId(importeeId.replace(matchedEntry.find, matchedEntry.replacement));
            var customResolver = getCustomResolver(matchedEntry, options);
            if (customResolver) {
                return customResolver.call(this, updatedId, importerId);
            }
            return this.resolve(updatedId, importer, { skipSelf: true }).then(function (resolved) {
                var finalResult = resolved;
                if (!finalResult) {
                    finalResult = { id: updatedId };
                }
                return finalResult;
            });
        }
    };
}

export default alias;
